//-------------------------------------------------------------------
//	File:		main_r00.c
//	
//
// Written By:		Nicola Ferrarese						
// This code implements a bipolar stepper motor control.
// SN754410 device and PIC16F873 MCU are used.
// Half step mode controll are used.
// The code was compiled using the HI-TECH PICC compiler v. 8.02PL1 Demo.
// Project > Build Options... > Project > PICC Compiler, check Enable assembler optimization
// 
//---------------------------------------------------------------------

//---------------------------------------------------------------------
//	HEADER FILE INCLUDES
//---------------------------------------------------------------------
#include <pic1687x.h>		// Register definitions
#include <pic.h>			// Header file 
#include "delay.h"			// Delay software routine 
#include "com_types.h"		// Common type definitions

#define	rpmmax	30600							// 30600	eq. 306.00 rpm
#define	rpmmin	600								// 600		eq.   6.00 rpm
#define	step	200								// motor's number step per revolution
#define velocityslope	(rpmmax-rpmmin)/120		// 120 is ADC Span for one direction (CC,CCW)=(0..120,135..255)

//	rps=rpm/60;									// revolution per second
// 	Ts=1/rps=60/rpm								// Period of a revolution in sec.
//	Tus=10^6 Ts=(10^6)*60/rpm					// Period of revolution in us
//	Tw=Tus/step=(10^6)*60/(rpm*step)			// Period wave in us
//  Please note that rpm variables are *100 value: Ex. rpm=10000 => 100.00 revolution per minute 
//	
//	Tw=(10^6)*60/[rpm/100)*step]=600000000/step
// 	Ex. step=200 => rpm2us=30000000
#define	rpm2us	30000000						// rpm to step period conversion constant

//---------------------------------------------------------------------
//	Macros
//---------------------------------------------------------------------
__CONFIG( UNPROTECT & DEBUGDIS & WRTDIS & DUNPROT & LVPDIS & BORDIS &
		 PWRTEN & WDTDIS & XT );
									

//---------------------------------------------------------------------
//	Port Pin Designations
//---------------------------------------------------------------------
static bit LEDR	@ BITNUM(PORTC, 0);
static bit LEDV	@ BITNUM(PORTC, 1);
static bit EN1	@ BITNUM(PORTB, 0); 
static bit PH1	@ BITNUM(PORTB, 1);
static bit EN2	@ BITNUM(PORTB, 2); 
static bit PH2	@ BITNUM(PORTB, 3);
static bit SW1	@ BITNUM(PORTC, 4);
static bit SW2	@ BITNUM(PORTC, 5);


#define	ENABLE1		EN1=0				// enable driver1
#define DISABLE1	EN1=1				// disable driver1
#define	ENABLE2		EN2=0				// enable driver2
#define DISABLE2	EN2=1				// disable driver2

//---------------------------------------------------------------------
//	Variable declarations
//---------------------------------------------------------------------
static unsigned char
adc8,									// unsigned 8bit ADC result [0..255]
ind,									// step counter, free count up or down
opmode,									// operation mode
dir;									// direction 0=CW; 1=CCW

static unsigned int
rpm,									// rpmx100 ex. 18000->180.00
ts,										// step pulse lenght us
ths,									// half step pulse lenght us
timer1_init;							// timer1 value reloaded every interrput			

//---------------------------------------------------------------------
// Function Prototypes Dichiarazione
//---------------------------------------------------------------------			
static void SetUp(void);						// Initialize ports/peripherals 
static void InitVar(void);						// Initialize variable
static void interrupt Isr(void);				// Interrupt Service Routine
static unsigned char ADC_Measure8(void);		// Make 8bit ADC conversion
static unsigned int	calcrpm(unsigned char);		// Calculate rpm*100
static unsigned int calctw(unsigned int);		// Calculate step wave duration
static unsigned char startok(void);				// Check to system start with motor halded


//---------------------------------------------------------------------
// Main program
//---------------------------------------------------------------------
main()
{
SetUp();										// Initialize PORT and Peripheal
InitVar();										// Initialize variabile
LEDV=LEDON;
LEDR=LEDON;
while (startok());								// system starts only if potentiometer's cursor is on mid position	(rpm=0)			

while(1)										// This is the main program loop 
	{
	if(T0IF)									// Polling tmr0 overflow
		{										// every 256x256us timer0 rolloff,256 prescler
		adc8=ADC_Measure8();					// measure set froma ext. potemtiometer
		rpm=calcrpm(adc8);						// calculate rpm profile
		if(rpm==0)
			{									// Motor halted
			TMR1IE=0;							// disable timer1 interrupt
			DISABLE1;
			DISABLE2;
			timer1_init=0;
			LEDR=LEDOFF;
			}
		else
			{									// Motor run						
			ts=calctw(rpm);						// calculate step wave pulse width
			ths=ts/2;							// calculate half step pulse width																
			timer1_init=0xFFFF-ths;				// calculate preset for timer1
			TMR1IE=1;							// Enable timer1 overflow interrupt
			LEDR=LEDON;
			}
		LEDV=!LEDV;
		T0IF=0;
		}										//
	}											// end while
}												// end main										

	
//---------------------------------------------------------------------
//	Setup() initializes peripheral registers
//---------------------------------------------------------------------
static void SetUp(void)
{
						// SetUp I/O Port configuration 0=Output, 1=Input
TRISA = 0x01;			// PORTA input output configuration 
PORTA = 0x00;			// Clear PORTA -  All pins Low
TRISB = 0x00;			// PORTB input output configuration
PORTB = 0b00000101;		// PORTB init
TRISC = 0x30;			// PORTC input output configuration
PORTC = 0;				// Clear PORTC

						// SetUp AD Converter
ADCON0 	= 0b11000001;	// ADC RCclock, turn ADC on
ADCON1 	= 0b00001110;	// ADC result left justified, RA0/AN0 only is an analog input

						// SetUp Timer0
T0CS=0;					// TMRO internal CLK source
PSA=0;					// Prescaler assigned to TMR0 module
PS2=1;					// Prescaler
PS1=1;					// rate 
PS0=1;					// is 1:256
RBPU=0;					// Enable PORTB pull-ups internal resistor

						// SetUp interrupt
INTCON=0;				// Disable all interrupts		
PIE1=0x00;				// Disable all other interrupt source
PIR1=0x00;				// Clear all peripheral interrupt flag
TMR1L=0x00;				// Reset Timer1 register pair
TMR1H=0x00;				// Reset Timer1 register pair
TMR1ON=1;				// Turn Timer1 on
TMR1IE=0;     			// Start with timer1 interrupt disabled
PEIE=1;					// Enable all un masked peripheral interrupts
GIE=1;					// Enable Global Interrupts
}


//---------------------------------------------------------------------
//	InitVar() initializes program variables 
//---------------------------------------------------------------------

static void InitVar(void)
{
DISABLE1;
DISABLE2;
}


//---------------------------------------------------------------------
//	Isr() Interrupt service routine
//---------------------------------------------------------------------

static void interrupt Isr(void)
{										// save status, bit overhead						
TMR1L=timer1_init&0x00FF;				// Load 16 bit tmr1 value 
TMR1H=(unsigned int)timer1_init>>8;		//

switch(ind%8)			
	{
	case 0:				// I,0
		PH1=1;
		ENABLE1;	
		DISABLE2;	
	break;

	case 1:				// I,I
		PH1=1;
		ENABLE1;
		PH2=1;
		ENABLE2;
	break;

	case 2:				// 0,I
		DISABLE1;
		PH2=1;
		ENABLE2;		
	break;

	case 3:				// -I,I
		PH1=0;
		ENABLE1;
		PH2=1;
		ENABLE2;
	break;

	case 4:				// -I,0			
		PH1=0;
		ENABLE1;
		DISABLE2;
	break;

	case 5:				// -I,-I
		PH1=0;
		ENABLE1;
		PH2=0;
		ENABLE2;
	break;

	case 6:				// 0,-I
		DISABLE1;
		PH2=0;
		ENABLE2;
	break;

	case 7:				// I,-I
		PH1=1;
		ENABLE1;
		PH2=0;
		ENABLE2;
	break;
	
	default:
		DISABLE1;
		DISABLE2;
	break;
	}						// end switch

if(dir)	ind++;				// next step
else	ind--;

TMR1IF=0; 					// clear Timer1 interrupt Flag
}							// end Isr




//---------------------------------------------------------------------
//	8bit Analog to Digital conversion
//---------------------------------------------------------------------
static unsigned char ADC_Measure8(void)
{
ADGO = 1;  	                    // start ADC conversion
while(ADGO) {};					// Wait end of conversion					
return ADRESH;					// 8bit result, left justified
}



//---------------------------------------------------------------------
//	Calculate rpm*100 and set direction
//---------------------------------------------------------------------
static unsigned int	calcrpm(unsigned char adc8)
{
// return rpm value*100 calculated on adc8 readed value;
// table below are implemented: 
// adc8=0 		-> rpmmax	dir=0
// adc8=120 		-> rpmmin	dir=0
// 120<adc8<135	-> rpm=0
// adc8=135		-> rpm_min	dir=1
// adc8=255		-> rpmmax	dir=1

unsigned int retval;
if(adc8<121) 
	{
	dir=0;
	retval=(unsigned int) (rpmmin+velocityslope*(120-adc8));
	}
else
	{
	if(adc8>134)
		{
		dir=1;
		retval=(unsigned int) (rpmmin+velocityslope*(adc8-135));
		}
	else retval=0;
	}
return retval;
}


//---------------------------------------------------------------------
//	Calculate Twait [us]
//---------------------------------------------------------------------
static unsigned int calctw(unsigned int rpm)
{
// convert rpm to a time lenght for a step controll
// an [us] value are returned.
unsigned int retval;
if(rpm) retval=(unsigned long) (rpm2us/rpm);
else retval=0xFFFF;
return retval;
}
	

//---------------------------------------------------------------------
//	Check to Start with motor halted
//---------------------------------------------------------------------
static char startok(void)
{
char retval;
DelayMs(2);
adc8=ADC_Measure8();
if((adc8<121)||(adc8>134)) retval=255;
else retval=0;
return retval;
}

//---------------------------------------------------------------------
//	Check to Switch to set opmode
//---------------------------------------------------------------------
static char SetOpMode(void)
{
char temp;
temp=PORTB;
temp^=0xFF;
temp&=0b00110000;
temp>>4;
return temp;
}



// 	WAVE MODE
// 	Step   Ph1 Ph2 
// 	1 	    I   0 
// 	2       0   I 
// 	3      -I   0 
// 	4       0  -I 
//
// 	TWO PHASE ON
//	Step   Ph1	Ph2 
//	1 	    I    I 
//	2      -I    I 
//	3      -I   -I 
//  4       I   -I
//
//	HALF STEP
//	Step   Ph1	Ph2
//	1		I	 0
//  2       I    I 
//  3       0    I 
//  4      -I    I 
//  5      -I    0 
//  6      -I   -I 
//  7       0   -I 
//  8       I   -I 
 







